<?php
/**
 * @file
 * Subs admin UI.
 */

/**
 * Subscription Type UI controller.
 */
class SubsTypeUIController extends EntityDefaultUIController {

  /**
   * Overrides hook_menu() defaults.
   */
  public function hook_menu() {
    $items = parent::hook_menu();
    $items[$this->path]['description'] = 'Manage subscription types, including fields.';
    return $items;
  }

}

/**
 * Form callback.
 * Generates the subscription type edit form.
 */
function subs_type_form($form, &$form_state, $subscription_type, $op = 'edit') {
  if ($op == 'clone') {
    $subscription_type->label .= ' (cloned)';
    $subscription_type->type = '';
  }

  $form['label'] = array(
    '#title' => t('Label'),
    '#type' => 'textfield',
    '#default_value' => $subscription_type->label,
    '#description' => t('The human-readable name of this subscription type.'),
    '#required' => TRUE,
    '#size' => 30,
  );

  // Machine-readable type name.
  $form['type'] = array(
    '#type' => 'machine_name',
    '#default_value' => isset($subscription_type->type) ? $subscription_type->type
        : '',
    '#maxlength' => 32,
    '#disabled' => $subscription_type->isLocked() && $op != 'clone',
    '#machine_name' => array(
      'exists' => 'subs_get_types',
      'source' => array('label'),
    ),
    '#description' => t('A unique machine-readable name for this subscription type. It must only contain lowercase letters, numbers, and underscores.'),
  );

  $length = isset($subscription_type->length) ? $subscription_type->length : 0;
  $days = _subs_admin_secs_to_days($length);

  $form['length'] = array(
    '#title' => t('Length'),
    '#type' => 'textfield',
    '#description' => t('The length of this subscription type, in days.'),
    '#default_value' => $days,
    '#size' => 6,
  );

  $length = isset($subscription_type->grace) ? $subscription_type->grace : 0;
  $days = _subs_admin_secs_to_days($length);

  $form['grace'] = array(
    '#title' => t('Grace period length'),
    '#type' => 'textfield',
    '#description' => t("The length of this subscription type's grace period, in days."),
    '#default_value' => $days,
    '#size' => 6,
  );

  $form['default_status'] = array(
    '#title' => t('Default subscription status'),
    '#type' => 'select',
    '#options' => subs_status_options_list(),
    '#default_value' => isset($subscription_type->default_status) ? $subscription_type->default_status
        : SUBS_PENDING,
    '#description' => t("The default status for new subscriptions of this type."),
  );

  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#default_value' => isset($subscription_type->description) ? $subscription_type->description
        : '',
    '#description' => t('A brief description of this subscription type.'),
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save subscription type'),
    '#weight' => 40,
  );

  if (!$subscription_type->isLocked() && $op != 'add' && $op != 'clone') {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete subscription type'),
      '#weight' => 45,
      '#limit_validation_errors' => array(),
      '#submit' => array('subs_type_form_submit_delete')
    );
  }
  return $form;
}

/**
 * Validation callback.
 * Handles validating subscription types.
 *
 * @see subs_type_form()
 */
function subs_type_form_validate(&$form, &$form_state) {
  // Length must be numeric.
  if (!is_numeric($form_state['values']['length'])) {
    form_set_error('length', t('Invalid subscription length.'));
  }
  // Length must be positive.
  elseif ($form_state['values']['length'] < 0) {
    form_set_error('length', t('Subscription length must be greater than or equal to zero.'));
  }
  // We're good, convert to seconds.
  else {
    form_set_value($form['length'], _subs_admin_days_to_secs($form_state['values']['length']), $form_state);
  }

  // Grace period must be numeric.
  if (!is_numeric($form_state['values']['grace'])) {
    form_set_error('grace', t('Invalid subscription grace period length.'));
  }
  // Grace period must be positive.
  elseif ($form_state['values']['grace'] < 0) {
    form_set_error('grace', t('Subscription grace period length must be greater than or equal to zero.'));
  }
  // We're good, convert to seconds.
  else {
    form_set_value($form['grace'], _subs_admin_days_to_secs($form_state['values']['grace']), $form_state);
  }

  // Default status must be valid.
  $options = subs_status_options_list();
  if (!array_key_exists($form_state['values']['default_status'], $options)) {
    form_set_error('default_status', t('Invalid default status.'));
  }
}

/**
 * Submit callback.
 * Handles saving subscription types.
 *
 * @see subs_type_form()
 */
function subs_type_form_submit(&$form, &$form_state) {
  $subscription_type = entity_ui_form_submit_build_entity($form, $form_state);
  // Save and go back.
  $subscription_type->save();
  $form_state['redirect'] = 'admin/structure/subscriptions';
}

/**
 * Submit callback.
 * Handles deleting subscription types.
 *
 * @see subs_type_form()
 */
function subs_type_form_submit_delete(&$form, &$form_state) {
  $form_state['redirect'] = 'admin/structure/subscriptions/manage/' . $form_state['subs_type']->type . '/delete';
}

/**
 * Converts seconds to days.
 *
 * @param int
 *   The number of seconds to convert.
 * @return int
 *   The number of corresponding days.
 */
function _subs_admin_secs_to_days($seconds) {
  // 1 day = 86400 seconds
  return floor($seconds / 86400);
}

/**
 * Converts days to seconds.
 *
 * @param int
 *   The number of days to convert.
 * @return int
 *   The number of corresponding seconds.
 */
function _subs_admin_days_to_secs($days) {
  return $days * 86400;
}