<?php
/**
 * @file
 * Subs Drush integration.
 */

/**
 * Implements hook_drush_command().
 */
function subs_drush_command() {
  return array(
    // Utility commands.
    'subs-cron' => array(
      'description' => 'Execute subscription cron script.',
    ),
    'subs-load' => array(
      'description' => 'Load and view a subscription',
      'aliases' => array('subs-view'),
      'arguments' => array(
        'id' => 'The id of the subscription to load.',
      ),
    ),
    'subs-delete' => array(
      'description' => 'Delete a subscription',
      'aliases' => array('subs-del'),
      'arguments' => array(
        'id' => 'The id of the subscription to delete.',
      ),
    ),
    // Change subscription status commands.
    'subs-pending' => array(
      'description' => 'Make a subscription pending.',
      'arguments' => array(
        'id' => 'The id of the subscription to make pending.',
      ),
    ),
    'subs-active' => array(
      'description' => 'Activate a subscription.',
      'arguments' => array(
        'id' => 'The id of the subscription to activate.',
      ),
    ),
    'subs-grace' => array(
      'description' => 'Change a subscription to grace period.',
      'arguments' => array(
        'id' => 'The id of the subscription to change to grace period.',
        'extend' => "true/false. Whether to extend the subscription's end date by the grace period length.",
      ),
    ),
    'subs-expire' => array(
      'description' => 'Expire a subscription.',
      'arguments' => array(
        'id' => 'The id of the subscription to expire.',
      ),
    ),
    'subs-cancel' => array(
      'description' => 'Cancel a subscription.',
      'arguments' => array(
        'id' => 'The id of the subscription to cancel.',
      ),
    ),
  );
}

/**
 * Implements hook_drush_help().
 */
function subs_drush_help($section) {
  switch ($section) {
    case 'drush:subs-cron':
      return dt("Execute subscription cron script. Use this to execute the subscription cron function independently from Drupal's cron.");
    case 'drush:subs-load':
      return dt('Load and view a subscription.');
    case 'drush:subs-delete':
      return dt('Delete a subscription.');
    case 'drush:subs-pending':
      return dt('Make a subscription pending.');
    case 'drush:subs-active':
      return dt('Activate a subscription.');
    case 'drush:subs-grace':
      return dt('Change a subscription to grace period.');
    case 'drush:subs-expire':
      return dt('Expire a subscription.');
    case 'drush:subs-cancel':
      return dt('Cancel a subscription.');
  }
}

########################
### Utility Commands ###
########################

/**
 * Drush command.
 * Execute subscription cron script.
 *
 * @see subs_drush_command()
 */
function drush_subs_cron() {
  subs_cron();
  drush_print(dt('Subs cron executed!'));
}

/**
 * Drush command.
 * Load and view a subscription.
 *
 * @param int
 *   The $id of the subscription to load.
 */
function drush_subs_load($id) {
  if ($subscription = subs_load($id)) {
    drush_print(dt('Subscription loaded successfully!') . "\n");
    print_r($subscription);
    drush_print("\n");
  }
  else {
    drush_print(dt('Failed to load the subscription. Invalid subscription id.'));
  }
}

/**
 * Drush command.
 * Delete a subscription.
 *
 * @param int
 *   The $id of the subscription to delete.
 */
function drush_subs_delete($id) {
  if ($subscription = subs_load($id)) {
    if (drush_confirm(dt('Are you sure you want to delete this subscription?'))) {
      subs_delete($subscription);
      drush_print(dt('Subscription deleted successfully!'));
    }
  }
  else {
    drush_print(dt('Failed to load the subscription. Invalid subscription id.'));
  }
}

##############################
### Change Status Commands ###
##############################

/**
 * Drush command.
 * Make a subscription pending.
 *
 * @param int
 *   The $id of the subscription to change.
 */
function drush_subs_pending($id) {
  _drush_subs_status($id, 'subs_set_pending');
}

/**
 * Drush command.
 * Activate a subscription.
 *
 * @param int
 *   The $id of the subscription to change.
 */
function drush_subs_active($id) {
  _drush_subs_status($id, 'subs_set_active');
}

/**
 * Drush command.
 * Change a subscription to grace period.
 *
 * @param int
 *   The $id of the subscription to change.
 * @param boolean
 *   Whether to extend the subscription's end date by the grace period length.
 */
function drush_subs_grace($id, $extend) {
  _drush_subs_status($id, 'subs_set_grace', (bool) $extend);
}

/**
 * Drush command.
 * Expire a subscription.
 *
 * @param int
 *   The $id of the subscription to change.
 */
function drush_subs_expire($id) {
  _drush_subs_status($id, 'subs_set_expired');
}

/**
 * Drush command.
 * Cancel a subscription.
 *
 * @param int
 *   The $id of the subscription to change.
 */
function drush_subs_cancel($id) {
  _drush_subs_status($id, 'subs_set_cancelled');
}

/**
 * Helper function.
 * Executes status operations on subscriptions.
 *
 * @param int
 *   The subscription $id we want to change.
 * @param string
 *   The operation $callback we want to execute.
 * @param mixed
 *   Any extra parameter to pass to the callback.
 */
function _drush_subs_status($id, $callback, $extra = NULL) {
  if ($subscription = subs_load($id)) {
    // Try to execute operation callback.
    if ($callback($subscription, $extra)) {
      drush_print(dt('Subscription status changed!'));
    }
    else {
      drush_print(dt('Failed to change subscription status.'));
    }
  }
  else {
    drush_print(dt('Failed to load the subscription. Invalid subscription id.'));
  }
}