<?php
/**
 * @file
 * Subs entity classes.
 */

/**
 * The class used for subscription entities.
 */
class Subs extends Entity {

  /**
   * The subscription id.
   *
   * @var integer
   */
  public $sid;

  /**
   * The name of the subscription type.
   *
   * @var string
   */
  public $type;

  /**
   * The user id of the applicant.
   *
   * @var integer
   */
  public $uid;

  /**
   * The Unix timestamp when the subscription was created.
   *
   * @var integer
   */
  public $created;

  /**
   * The Unix timestamp when the subscription was most recently saved.
   *
   * @var integer
   */
  public $changed;

  /**
   * The Unix timestamp when the subscription started.
   *
   * @var integer
   */
  public $date_start;

  /**
   * The Unix timestamp when the subscription will end.
   *
   * @var integer
   */
  public $date_end;

  /**
   * The subscription status.
   *
   * @var boolean
   */
  public $status;

  public function __construct($values = array()) {
    if (isset($values['type']) && is_object($values['type'])) {
      $values['type'] = $values['type']->type;
    }
    parent::__construct($values, 'subs');
  }

  /**
   * Returns the full url() for the subscription.
   */
  public function url() {
    $uri = $this->uri();
    return url($uri['path'], $uri);
  }

  /**
   * Returns the drupal path to this subscription.
   */
  public function path() {
    $uri = $this->uri();
    return $uri['path'];
  }

  protected function defaultUri() {
    return array(
      'path' => 'subscription/' . $this->sid,
    );
  }

  protected function defaultLabel() {
    $username = isset($this->uid) ? user_load($this->uid)->name : 'username';
    $type = entity_label('subs', subs_get_types($this->type));
    return t('@type subscription by @username', array('@type' => $type, '@username' => $username));
  }

  public function save() {
    // Care about setting created and changed values. But do not automatically
    // set a created values for already existing subscriptions.
    if (empty($this->created)) {
      $this->created = REQUEST_TIME;
    }
    $this->changed = REQUEST_TIME;

    if ($return = parent::save()) {
      // Update or set the static cache from subs_load_by_user().
      $cache = &drupal_static('subs_load_by_user', array());
      $cache[$this->uid] = $this->sid;
      return $return;
    }

    return FALSE;
  }

}

/**
 * Use a separate class for subscription types so we can specify some defaults
 * modules may alter.
 */
class SubsType extends Entity {

  public $type;
  public $label;
  public $weight = 0;

  public function __construct($values = array()) {
    parent::__construct($values, 'subs_type');
  }

  /**
   * Returns whether the subscription type is locked and thus may not be deleted or renamed.
   *
   * Subscription types provided in code are automatically treated as locked, as well
   * as any fixed subscription type.
   */
  public function isLocked() {
    return isset($this->status) && empty($this->is_new) && (($this->status & ENTITY_IN_CODE) || ($this->status & ENTITY_FIXED));
  }

}

/**
 * The controller class for subscription entities.
 */
class SubsController extends EntityAPIController {

  /**
   * Add properties as pseudo-fields.
   */
  public function buildContent($entity, $view_mode = 'full', $langcode = NULL, $content = array()) {
    // @TODO figure out how to handle labels (inline, above, hidden).
    // @TODO allow different formatters for date_start, date_end and uid.

    // Make User property themed like a default field.
    $user = user_load($entity->uid);
    $content['uid'] = array(
      '#theme' => 'field',
      '#title' => t('User'),
      '#access' => TRUE,
      '#label_display' => 'inline',
      '#view_mode' => $view_mode,
      '#language' => LANGUAGE_NONE,
      '#field_name' => 'field_fake_description',
      '#field_type' => 'text',
      '#entity_type' => 'subs',
      '#bundle' => $entity->type,
      '#items' => array(array('value' => $entity->uid)),
      '#formatter' => 'text_default',
      0 => array('#markup' => l($user->name, 'user/' . $entity->uid)),
    );

    // Make Start Date property themed like a default field.
    $content['date_start'] = array(
      '#theme' => 'field',
      '#title' => t('Start Date'),
      '#access' => TRUE,
      '#label_display' => 'inline',
      '#view_mode' => $view_mode,
      '#language' => LANGUAGE_NONE,
      '#field_name' => 'field_fake_description',
      '#field_type' => 'text',
      '#entity_type' => 'subs',
      '#bundle' => $entity->type,
      '#items' => array(array('value' => $entity->date_start)),
      '#formatter' => 'text_default',
      0 => array('#markup' => format_date($entity->date_start, 'medium')),
    );

    // Make End Date property themed like a default field.
    // @TODO make this text a setting?
    $date_end = empty($entity->date_end) ? t('Indefinite') : format_date($entity->date_end, 'medium');
    $content['date_end'] = array(
      '#theme' => 'field',
      '#title' => t('End Date'),
      '#access' => TRUE,
      '#label_display' => 'inline',
      '#view_mode' => $view_mode,
      '#language' => LANGUAGE_NONE,
      '#field_name' => 'field_fake_description',
      '#field_type' => 'text',
      '#entity_type' => 'subs',
      '#bundle' => $entity->type,
      '#items' => array(array('value' => $entity->date_end)),
      '#formatter' => 'text_default',
      0 => array('#markup' => $date_end),
    );

    // Make Status property themed like a default field.
    $status_options = subs_status_options_list();
    $content['status'] = array(
      '#theme' => 'field',
      '#title' => t('Status'),
      '#access' => TRUE,
      '#label_display' => 'inline',
      '#view_mode' => $view_mode,
      '#language' => LANGUAGE_NONE,
      '#field_name' => 'field_fake_description',
      '#field_type' => 'text',
      '#entity_type' => 'subs',
      '#bundle' => $entity->type,
      '#items' => array(array('value' => $entity->status)),
      '#formatter' => 'text_default',
      0 => array('#markup' => check_plain($status_options[$entity->status])),
    );

    // Make Type property themed like a default field.
    $content['type'] = array(
      '#theme' => 'field',
      '#title' => t('Type'),
      '#access' => TRUE,
      '#label_display' => 'inline',
      '#view_mode' => $view_mode,
      '#language' => LANGUAGE_NONE,
      '#field_name' => 'field_fake_description',
      '#field_type' => 'text',
      '#entity_type' => 'subs',
      '#bundle' => $entity->type,
      '#items' => array(array('value' => $entity->type)),
      '#formatter' => 'text_default',
      0 => array('#markup' => entity_label('subs', subs_get_types($entity->type))),
    );

    return parent::buildContent($entity, $view_mode, $langcode, $content);
  }

}